==> rbamtools_2.3.5.tar.gz: rbamtools/src/gapSiteListList.h <==
/*
 * gapSiteListList.h
 *
 *  Created on: 11.01.2013
 *      Author: kaisers
 */

#ifndef GAPSITELISTLIST_H_
#define GAPSITELISTLIST_H_

#include "gapSiteList.h"

typedef struct site_ll_element
{
	site_list *l;
	struct site_ll_element *next;
	struct site_ll_element *last;
}site_ll_element;

inline site_ll_element * site_ll_element_init(site_list *l)
{
	site_ll_element* el=calloc(sizeof(site_ll_element),1);
	el->l=l;
	return el;
}

inline void site_ll_element_destroy(site_ll_element *el)
{
	site_list_destroy(el->l);
	free(el);
}


typedef struct site_ll
{
	struct site_ll_element *first;
	struct site_ll_element *last;
	struct site_ll_element *curr;
	unsigned size;
} site_ll;

inline site_ll * site_ll_init()
{
	site_ll * sll=calloc(sizeof(site_ll),1);
	return sll;
}

void site_ll_add_site_list(site_ll *l,site_list *sl)
{
	site_ll_element *el=site_ll_element_init(sl);
	if(l->size==0)
	{
		l->first=el;
		l->last=el;
		l->size=1;
	}
	else
	{
		el->last=l->last;
		l->last->next=el;
		l->last=el;
		++(l->size);
	}
}

void site_ll_destroy(site_ll *l)
{
	if(l==0)
		return;
	site_ll_element *el;
	while(l->size>0)
	{
		el=l->first;
		//printf("[site_ll_destroy] refid %u\n",el->l->refid);
		if(l->size>1)
			l->first=l->first->next;
		--(l->size);
		site_ll_element_destroy(el);
	}
	free(l);
}

site_list * site_ll_get_curr_site_list_pp(site_ll *l)
{
	if(l->curr==0)
		return 0;
	site_list* sl=l->curr->l;
	l->curr=l->curr->next;
	return sl;
}

void site_ll_set_curr_first(site_ll *l) { l->curr=l->first; }

pos_type sum_ll_sizes(const site_ll *l)
{
	if(l->size==0)
		return 0;

	pos_type size=0;
	unsigned i;
	site_ll_element *el=l->first;
	size=el->l->size;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		size+=el->l->size;
	}
	return size;
}

pos_type get_nAligns(const site_ll *l)
{
	if(l->size==0)
		return 0;
	pos_type nAligns=0;
	unsigned i;
	site_ll_element *el=l->first;
	nAligns=el->l->nAligns;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		nAligns+=el->l->nAligns;
	}
	return nAligns;
}

pos_type get_nGapAligns(const site_ll *l)
{
	pos_type nGapAligns=0;
	unsigned i;
	site_ll_element *el=l->first;
	nGapAligns=el->l->nGapAligns;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		nGapAligns+=el->l->nGapAligns;
	}
	return nGapAligns;
}


#endif /* GAPSITELISTLIST_H_ */

==> rbamtools_2.4.0.tar.gz: rbamtools/src/gapSiteListList.h <==
/*
 * gapSiteListList.h
 *
 *  Created on: 11.01.2013
 *      Author: kaisers
 */

#ifndef GAPSITELISTLIST_H_
#define GAPSITELISTLIST_H_

#include <R.h>
#include "gapSiteList.h"

typedef struct site_ll_element
{
	site_list *l;
	struct site_ll_element *next;
	struct site_ll_element *last;
}site_ll_element;

static R_INLINE site_ll_element * site_ll_element_init(site_list *l)
{
	site_ll_element* el=calloc(sizeof(site_ll_element),1);
	el->l=l;
	return el;
}

static R_INLINE void site_ll_element_destroy(site_ll_element *el)
{
	site_list_destroy(el->l);
	free(el);
}


typedef struct site_ll
{
	struct site_ll_element *first;
	struct site_ll_element *last;
	struct site_ll_element *curr;
	unsigned size;
} site_ll;

site_ll * site_ll_init()
{
	site_ll * sll=calloc(sizeof(site_ll),1);
	return sll;
}

void site_ll_add_site_list(site_ll *l,site_list *sl)
{
	site_ll_element *el=site_ll_element_init(sl);
	if(l->size==0)
	{
		l->first=el;
		l->last=el;
		l->size=1;
	}
	else
	{
		el->last=l->last;
		l->last->next=el;
		l->last=el;
		++(l->size);
	}
}

void site_ll_destroy(site_ll *l)
{
	if(l==0)
		return;
	site_ll_element *el;
	while(l->size>0)
	{
		el=l->first;
		//printf("[site_ll_destroy] refid %u\n",el->l->refid);
		if(l->size>1)
			l->first=l->first->next;
		--(l->size);
		site_ll_element_destroy(el);
	}
	free(l);
}

site_list * site_ll_get_curr_site_list_pp(site_ll *l)
{
	if(l->curr==0)
		return 0;
	site_list* sl=l->curr->l;
	l->curr=l->curr->next;
	return sl;
}

void site_ll_set_curr_first(site_ll *l) { l->curr=l->first; }

pos_type sum_ll_sizes(const site_ll *l)
{
	if(l->size==0)
		return 0;

	pos_type size=0;
	unsigned i;
	site_ll_element *el=l->first;
	size=el->l->size;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		size+=el->l->size;
	}
	return size;
}

pos_type get_nAligns(const site_ll *l)
{
	if(l->size==0)
		return 0;
	pos_type nAligns=0;
	unsigned i;
	site_ll_element *el=l->first;
	nAligns=el->l->nAligns;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		nAligns+=el->l->nAligns;
	}
	return nAligns;
}

pos_type get_nGapAligns(const site_ll *l)
{
	pos_type nGapAligns=0;
	unsigned i;
	site_ll_element *el=l->first;
	nGapAligns=el->l->nGapAligns;
	for(i=1;i<l->size;++i)
	{
		el=el->next;
		nGapAligns+=el->l->nGapAligns;
	}
	return nGapAligns;
}


#endif /* GAPSITELISTLIST_H_ */
